# frozen_string_literal: true

require "ffi"       # Bindings to C libraries

require "http-parser/ext"       # Loads http-parser ext
require "http-parser/errors"    # http-parser error definitions
require "http-parser/types"     # http-parser data structures
require "http-parser/parser"    # the core http-parser abstraction

module HttpParser
end
